#!/bin/bash
# Copyright (c) 2017 Red Hat, Inc.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or
# implied.
# See the License for the specific language governing permissions and
# limitations under the License.

for lib in \
        os-client-config \
        keystoneauth \
        python-novaclient \
        python-keystoneclient \
        python-cinderclient \
        python-neutronclient \
        python-ironicclient \
        python-heatclient \
        python-designateclient \
        python-magnumclient
do
    egg=$(echo $lib | tr '-' '_' | sed 's/python-//')
    if [ -d /opt/stack/new/$lib ] ; then
        pip install -q -U -e "git+file:///opt/stack/new/$lib#egg=$egg"
    fi
done
